#as:
#ld: -shared -z norelro --hash-style=both
#objdump: -dr

.*:     file format .*

Disassembly of section .text:

[0-9a-f]+ <fn1>:
.*[pcalau12i|pcaddu12i].*
.*ld.[d|w].*
.*[pcalau12i|pcaddu12i].*
.*ld.[d|w].*
